# -*- coding: utf-8 -*-
import datetime
from HTMLParser import HTMLParseError

from django.contrib import admin, messages
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _
from django.db.models import Q
from django import forms
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.conf.urls import patterns, url
from django.conf import settings
from django.shortcuts import render_to_response
from django.template.context import RequestContext
from django.contrib.contenttypes.models import ContentType

from jmb.core.admin.options import ExtendibleModelAdmin, SettingsAdmin
from jmb.core.forms import DateWidget

from jmb.newsletter.models import Newsletter, MailingList, Settings, Subscriber, NewsletterTemplate
from jmb.newsletter.sender import Mailer, EmailObject
from jmb.newsletter.utils.importation import import_dispatcher
from jmb.newsletter import views as newsletter_views

admin.site.register(Settings, SettingsAdmin)

#######################################
#  NEWSLETTER ADMIN                   #
#######################################
class SubscriberAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("newsletter/css/admin_newsletter.css",)}

    list_display = (
        'get_status_icon','mailinglist', 'email', 'date_subscription', 'date_unsubscription',
        'is_test_contact','get_edit_icon', 'get_delete_icon',
        )
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_("Ordering"), {
            'fields' : ('ordering',),
            'classes': ('collapse',)
        }),
        (None, {'fields': (
            'mailinglist',
            ('date_subscription', 'date_unsubscription'),
            'email','is_test_contact',
            )}),
        )
    advanced_search_fields = (
        ('mailinglist',),
        ('email__icontains',),
        ('date_subscription__gte', 'date_subscription__lte'),
        ('date_unsubscription__gte', 'date_unsubscription__lte'),
#        ('unsubscriptions', 'is_test_contact'),
#        ('date_unsubscription__isnull', 'is_test_contact'),
        ('is_test_contact',),
    )
    actions = [
        'set_as_test',
        'create_mailinglist',
        'add_to_mailinglist',
        'send_newsletter',
        'unsubscribe_subscribers',
    ]

    ## ACTIONS
    def get_actions(self, request):
        actions = super(SubscriberAdmin, self).get_actions(request)
        del actions['set_active']
        del actions['set_disactive']
        return actions

    def set_as_test(self, request, queryset):
        """
        Set contact as test
        """
        for contact in queryset:
            try:
                contact.is_test_contact = True
                contact.save()
                messages.info(request, _('%s succesfully set as test.') % contact)
            except Exception, e:
                messages.error(request, 'Error: %s - %s' % (contact, e)
                               # _('Error: %s - %s') % (contact, e)
                )
    set_as_test.short_description =  _("Set Contacts as test")

    def create_mailinglist(self, request, queryset):
        """
        Creates a mailing list from selected contacts
        """
        when = str(datetime.datetime.now()).split('.')[0]
        new_mailing = MailingList(
            name = _('New mailinglist at %s') % when,
            description = _('New mailing list created in admin at %s') % when
        )
        new_mailing.save()
        for contact in queryset:
            ris = new_mailing.add_subscriber(contact)

        self.message_user(request, _('%s succesfully created.') % new_mailing)
        return HttpResponseRedirect(
            reverse(
                'admin:newsletter_mailinglist_change',
                args=[new_mailing.pk]
            )
        )
    create_mailinglist.short_description = _('Create a mailinglist')

    def unsubscribe_subscribers(self, request, queryset):
        """
        Unsubscribe subscribers
        """
        now=datetime.datetime.now()
        totale=queryset.count()
        queryset = queryset.filter(date_unsubscription__isnull=True)
        totale_modificabili=queryset.count()
        modificati=0
        errore=0
        gia_disiscritti= totale - totale_modificabili

        messages.error(request, _("Subscribers already unsubscribed: %s") % gia_disiscritti)
        for contact in queryset:
            try:
                contact.date_unsubscription=now
                contact.save()
                modificati+=1
            except Exception, e:
                errore+=1
                messages.error(request, 'Error: %s - %s' % (contact, e)
                               # _('Error: %s - %s') % (contact, e)
                )
        messages.info(request, _("Subscribers unsubscripted: %s") % modificati)
    unsubscribe_subscribers.short_description = _("Unsubscribe subscribers")


    class SetMailingListForm(forms.Form):
        _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
        _selected_all = forms.IntegerField(widget=forms.HiddenInput)
        mailing_list = forms.ModelChoiceField(MailingList.objects)

    def get_dict(self, dict):
        new_dict = {}
        for key in dict.keys():
            new_dict[str(key)] = str(dict[key])
        return new_dict

    def add_to_mailinglist(self, request, queryset):
        """
        Adds selected contacts to mailing list
        request.POST.getlist(admin.ACTION_CHECKBOX_NAME)
        request.POST.get('select_accross')  - 'seleziona tutto' flag della lista
        request.POST.get('action')
        request.POST.get('apply')
        request.POST.get('_selected_action') - lista oggetti selezionati
        request.POST.get('_selected_all') - 'selezionati tutti' campo nascosto del form

        http://www.jpichon.net/blog/2010/08/django-admin-actions-and-intermediate-pages/
        """

        form = None
        # verifies that all items are selected
        if 'apply' in request.POST:
            dict = self.get_dict(request.GET)

            if request.POST.get('_selected_all') == '1':
                # se sono stati selezionati tutti i contatti (anche delle pagine successive)
                # quindi occorre ricostruire il queryset

                queryset = Subscriber.objects.filter(**dict)

            form = self.SetMailingListForm(request.POST)
            if form.is_valid():
                mailing_list = form.cleaned_data['mailing_list']
                count = 0
                plural = ""
                for contact in queryset:
                    ris = mailing_list.add_subscriber(contact)
                    if not ris[0]:
                        messages.error(request, "%s: %s" % (contact.email, ris[1]))
                    else:
                        count += 1
                if count == 0:
                    messages.error(request, "0 contacts are successfully added to %s." % mailing_list)
                elif count == 1:
                    messages.info(request, "1 contact is successfully added to %s." % mailing_list)
                else:
                    messages.info(request, "%d contacts are successfully added to %s." % (
                        count, mailing_list
                        ))
                return HttpResponseRedirect(request.get_full_path())

        if not form:
            form = self.SetMailingListForm(
                initial= {
                    '_selected_action': request.POST.getlist(admin.ACTION_CHECKBOX_NAME),
                    '_selected_all':    request.POST.get('select_across')
                }
            )

        context = {'contacts': queryset, 'form': form,}
        return render_to_response(
            'admin/newsletter/contact/set_mailing_list.html',
            context, RequestContext(request)
        )
    add_to_mailinglist.short_description = _('Add to mailinglist')

    class SetNewsletterForm(forms.Form):
        _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
        _selected_all = forms.IntegerField(widget=forms.HiddenInput)
        newsletter = forms.ModelChoiceField(Newsletter.objects)

    def send_newsletter(self, request, queryset):
        """
        Sends newsletter to selected contacts
        """
        form = None
        # verifies that all items are selected
        if 'apply' in request.POST:
            dict = self.get_dict(request.GET)

            if request.POST.get('_selected_all') == '1':
                # se sono stati selezionati tutti i contatti (anche delle pagine successive)
                # quindi occorre ricostruire il queryset

                queryset = Subscriber.objects.filter(**dict)

            form = self.SetNewsletterForm(request.POST)
            if form.is_valid():
                newsletter = form.cleaned_data['newsletter']
                count = 0
                plural = ""
                mailer = Mailer(newsletter, recipients=queryset.all())
                try:
                    mailer.run()
                except HTMLParseError:
                    messages.error(request, _('Unable send newsletter, due to errors within HTML.'))
                self.message_user(request, _('%s succesfully sent.') % newsletter)
                return HttpResponseRedirect(request.get_full_path())

        if not form:
            form = self.SetNewsletterForm(
                initial= {
                    '_selected_action': request.POST.getlist(admin.ACTION_CHECKBOX_NAME),
                    '_selected_all':    request.POST.get('select_across')
                }
            )

        context = {'contacts': queryset, 'form': form,}
        return render_to_response(
            'admin/newsletter/contact/set_newsletter.html',
            context, RequestContext(request)
        )
    send_newsletter.short_description = _('Send Newsletter')


    ## URLS
    def get_urls(self):
        urls = super(SubscriberAdmin, self).get_urls()
        my_urls = patterns('',
            url(r'^import/$',
                self.admin_site.admin_view(self.importation),
                name='newsletter_contact_import'),
        )
        return my_urls + urls

    def importation(self, request):
        """
        Import contacts from a CSV or Excel file
        """
        opts = self.model._meta

        if request.FILES:
            source = request.FILES.get('source')
            inserted = import_dispatcher(source, request.POST['type'])
            print inserted
        #            if inserted:
        #                contacts_imported.send(sender=self, source=source,
        #                    type=request.POST['type'])
        #
        #            self.message_user(request, _('%s contacts succesfully imported.') % inserted)
        #
        context = {
            'title': _('Contact importation'),
            'opts': opts,
            'root_path': self.admin_site.root_path,
            'app_label': opts.app_label
        }

        return render_to_response(
            'admin/newsletter/contact/contact_import.html',
            context, RequestContext(request)
        )

##############################
# NEWSLETTER                 #
##############################
class NewsletterAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("newsletter/css/admin_newsletter.css",)}
     #   js = ('newsletter/js/admin_newsletter.js',)


    date_hierarchy = 'date_create'
    list_display = (
        'status', 'title', 'mailing_list', 'historic_link', 'statistics_link',
        'sending_date', 'get_edit_icon', 'get_delete_icon'
    )
    list_display_links = ('get_edit_icon',)
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_("Ordering"), {
            'fields' : ('ordering',),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'title', 'slug', 'content', 'content_web', 'template')
        }),
        (_('Receivers'), {
            'fields': ('mailing_list',),
        }),
        (_('Sending'), {
            'fields': ('sending_date',),
        }),
        (_("SEO settings"), {
            'fields' : ('page_title', 'meta_description', 'meta_keywords'),
            'classes': ('collapse',)
        }),
    )
    prepopulated_fields = {'slug': ('title',)}
    save_on_top = True
    advanced_search_fields = (('title',),)
    actions = [
        'send_newsletter_test',
        'make_ready_to_send',
        'make_cancel_sending',
        'reset_status_values',
    ]


    ## ACTIONS
    def get_actions(self, request):
        actions = super(NewsletterAdmin, self).get_actions(request)
        del actions['set_active']
        del actions['set_disactive']
        if not request.user.has_perm('newsletter.can_change_status'):
            del actions['make_ready_to_send']
            del actions['make_cancel_sending']
            del actions['reset_status_values']
        return actions

    def send_newsletter_test(self, request, queryset):
        """Send newsletter in test"""
        for newsletter in queryset:
            if newsletter.mailing_list.has_test_contact:
                mailer = Mailer(newsletter, test=True)
                try:
                    mailer.run()
                except HTMLParseError:
                    messages.error(request, _('Unable send newsletter, due to errors within HTML.'))
                    continue
                self.message_user(request, _('%s succesfully sent.') % newsletter.title)
            else:
                messages.warning(request, _('No test contacts assigned for %s.') % newsletter)
    send_newsletter_test.short_description = _('Send test newsletter')

    def make_ready_to_send(self, request, queryset):
        """Make newsletter ready to send"""
        queryset = queryset.filter(status=settings.NEWSLETTER_STATUS_DRAFT)
        for newsletter in queryset:
            newsletter.status = settings.NEWSLETTER_STATUS_WAITING
            newsletter.save()
        self.message_user(request, _('%s newletters are ready to send') % queryset.count())
    make_ready_to_send.short_description = _('Make ready to send')

    def make_cancel_sending(self, request, queryset):
        """Cancel the sending of newsletters"""
        queryset = queryset.filter(
            Q(status=settings.NEWSLETTER_STATUS_WAITING) |
            Q(status=settings.NEWSLETTER_STATUS_SENDING)
        )
        for newsletter in queryset:
            newsletter.status = Newsletter.CANCELED
            newsletter.save()
        self.message_user(request, _('%s newletters are cancelled') % queryset.count())
    make_cancel_sending.short_description = _('Cancel the sending')

    def reset_status_values(self, request, queryset):
        """
        Reset all Newsletters status values
        """
        count = queryset.count()
        queryset = queryset.filter(status=settings.NEWSLETTER_STATUS_DRAFT)
        filtered_count = queryset.count()
        if count != filtered_count:
            messages.error(request, _("You can reset only DRAFT newsletters"))
        newsletter_content_type=ContentType.objects.get_for_model(Newsletter)
        for newsletter in queryset:
            emails = EmailObject.objects.filter(
                content_type=newsletter_content_type,
                object_id=newsletter.id
            )
            messages.info(request, "%s: reset status values" % newsletter.title)
            emails.delete()
    reset_status_values.short_description = _("Reset status values")


    ## LIST DISPLAY METHODS
    def historic_link(self, newsletter):
        """Display link for historic"""
        newsletter_type = ContentType.objects.get_for_model(newsletter)
        if newsletter.get_email_mailing_status().count() and newsletter.slug:
            return u'<a href="%(href)s?email_object__content_type__id=%(n_type_id)s&email_object__object_id=%(id)s">%(title)s</a>' % {
                'href':         reverse('admin:mailer_mailingstatus_changelist'),
                'title':        _('View historic'),
                'n_type_id':    newsletter_type.id,
                'id':           newsletter.id
            }
        return _('Not available')
    historic_link.allow_tags = True
    historic_link.short_description = _('Historic')

    def statistics_link(self, newsletter):
        """Display link for statistics"""
        newsletter_type = ContentType.objects.get_for_model(newsletter)
        if newsletter.get_email_mailing_status().count() and newsletter.slug:
            return u"""
                <a href="%(href)s">
                    <img
                        src="%(url)sadmin/images/statistics.png"
                        alt="%(title)s" title="%(title)s"
                    />
                </a>""" % {
                'href':     reverse(
                    'admin:newsletter_newsletter_statistics',
                    args=[newsletter.slug]
                ),
                'url':      settings.MEDIA_URL,
                'title':    _('View statistics')
            }
        return _('Not available')
    statistics_link.allow_tags = True
    statistics_link.short_description = _('St')

    ## URLS
    def get_urls(self):
        urls = super(NewsletterAdmin, self).get_urls()
        my_urls = patterns('',
            url(
                r'^statistics/(?P<slug>[-\w]+)/$',
                self.admin_site.admin_view(newsletter_views.newsletter_statistics),
                name='newsletter_newsletter_statistics'
            ),

        )
        return my_urls + urls

##############################
# MAILING LIST               #
##############################
class MailingListAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("newsletter/css/admin_newsletter.css",)}

    date_hierarchy = 'date_create'
    list_display = (
        'get_status_icon','date_create', 'name',
        'subscriptions', 'test_subscriptions', 'unsubscriptions',
        'get_edit_icon','get_delete_icon'
        )
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_("Ordering"), {
            'fields' : ('ordering',),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status','name', 'slug','description',)
        }),
        (_("SEO settings"), {
            'fields' : ('page_title','meta_description','meta_keywords'),
            'classes': ('collapse',)
        }),
    )
    prepopulated_fields = {'slug': ('name',)}
    advanced_search_fields = (('name',),)
    save_on_top = True

admin.site.register(NewsletterTemplate)
admin.site.register(Subscriber, SubscriberAdmin)
admin.site.register(Newsletter, NewsletterAdmin)
admin.site.register(MailingList, MailingListAdmin)