# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from cms.models.pluginmodel import CMSPlugin
from cms.models import CMSPlugin

from jmb.newsletter.models import MailingList


class SubscriptionSettings(CMSPlugin):
    slug = models.SlugField(
        max_length=100,
        null= True, blank=True
    )
    classes = models.CharField(
        max_length=100,
        null= True, blank=True
    )
    form = models.CharField(
        max_length=100,
        choices=settings.NEWSLETTER_CMSPLUGIN_SUBSCRIPTION_FORM_CLASSES,
        default="jmb.newsletter.forms.AllMailingListSubscriptionForm",
    )
    sent_from = models.EmailField(
        _('Indirizzo mittente delle mail'),
        default="sender@example.org",
    )
    thanks = models.CharField(_('Messaggio di conferma'), max_length=200)
    form_template_name = models.CharField(
        max_length=100,
        choices=settings.NEWSLETTER_CMSPLUGIN_SUBSCRIPTION_FORM_TEMPLATES,
        default="newsletter/cms_plugins/subscription_form.html"
    )
    thanks_template_name = models.CharField(
        max_length=100,
        choices=settings.NEWSLETTER_CMSPLUGIN_SUBSCRIPTION_THANKS_TEMPLATES,
        default="newsletter/cms_plugins/subscription_thanks.html"
    )
    email_template_name = models.CharField(
        max_length=100,
        choices=settings.NEWSLETTER_CMSPLUGIN_SUBSCRIPTION_EMAIL_TEMPLATES,
        default="newsletter/cms_plugins/subscription_email.html"
    )
    tracking_url = models.URLField(
        max_length=200,
        null=True, blank=True,
        verbose_name=_("Tracking URL"),
        help_text=_("(Optional) URL for redirection after successful submit. Insert here the full URL for "
                    "tracking (with params).")
    )

    def __unicode__(self):
        return self.form_template_name

    class Meta:
        app_label = "newsletter"

class UnsubscriptionSettings(CMSPlugin):
    slug = models.SlugField(
        max_length=100,
        null= True, blank=True
    )
    classes = models.CharField(
        max_length=100,
        null= True, blank=True
    )
    form = models.CharField(
        max_length=100,
        choices=settings.NEWSLETTER_CMSPLUGIN_UNSUBSCRIPTION_FORM_CLASSES,
        default="jmb.newsletter.forms.MailingListUnsubscriptionForm",
    )
    sent_from = models.EmailField(
        _('Indirizzo mittente delle mail'),
        default="sender@example.org",
    )
    thanks = models.CharField(_('Messaggio di conferma'), max_length=200)
    form_template_name = models.CharField(
        max_length=100,
        choices=settings.NEWSLETTER_CMSPLUGIN_UNSUBSCRIPTION_FORM_TEMPLATES,
        default="newsletter/cms_plugins/unsubscription_form.html"
    )
    thanks_template_name = models.CharField(
        max_length=100,
        choices=settings.NEWSLETTER_CMSPLUGIN_UNSUBSCRIPTION_THANKS_TEMPLATES,
        default="newsletter/cms_plugins/unsubscription_thanks.html"
    )
    email_template_name = models.CharField(
        max_length=100,
        choices=settings.NEWSLETTER_CMSPLUGIN_UNSUBSCRIPTION_EMAIL_TEMPLATES,
        default="newsletter/cms_plugins/unsubscription_email.html"
    )
    def __unicode__(self):
        return self.form_template_name

    class Meta:
        app_label = "newsletter"

class NewsletterSettings(CMSPlugin):

    slugs_included = models.CharField(
        _("Slug(s) included"),
        max_length=150,
        blank=True,
        help_text=_("(Optional) Including newsletter slug(s), separated by ','")
    )
    mailinglist_slugs_included = models.CharField(
        _("Mailing list slug(s) included"),
        max_length=150,
        blank=True,
        help_text=_("(Optional) Including mailing list slug(s), separated by ','")
    )
    slugs_excluded = models.CharField(
        _("Slug(s) excluded"),
        max_length=150,
        blank=True,
        help_text=_("(Optional) Excluding newsletter slug(s), separated by ','")
    )
    mailinglist_slugs_excluded = models.CharField(
        _("Mailing list slug(s) excluded"),
        max_length=150,
        blank=True,
        help_text=_("(Optional) Excluding mailing list slug(s), separated by ','")
    )
    sending_date_begin = models.DateTimeField(
        _("Sending date begin"),
        blank=True,null=True,
        help_text=_('(Optional) Sending date filter  - lower limit'),
    )
    sending_date_end = models.DateTimeField(
        _("Sending date end"),
        blank=True, null=True,
        help_text=_('(Optional) Sending date filter  - upper limit'),
    )
    ordering = models.CharField(
        _("Ordering"),
        max_length=100,
        default='ordering, -sending_date, mailing_list',
        help_text=_("Newsletter ordering fields, separated by ','")
    )
    objects_per_page = models.IntegerField(
        _("Objects per page"),
        default = 10,
    )
    template_name = models.CharField(
        _("Template"),
        max_length=150,
        choices=settings.NEWSLETTER_LIST_CMSPLUGIN_TEMPLATES,
        default="newsletter/cms_plugins/newsletter_list.html",
    )

    class Meta:
        app_label = "newsletter"

class NewsletterSearchSettings(CMSPlugin):
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.NEWSLETTER_CMSPLUGIN_SEARCH_FORM_CLASSES,
        default="jmb.newsletter.forms.SearchNewsletterForm",
    )
    template_name = models.CharField(
        max_length=100,
        choices=settings.NEWSLETTER_CMSPLUGIN_SEARCH_TEMPLATES,
        default="newsletter/cms_plugins/search.html"
    )

    class Meta:
        app_label = "newsletter"

    def __unicode__(self):
        return self.template_name