# -*- coding: utf-8 -*-
from django.conf import settings
from django.utils.translation import ugettext as _
from django.views.generic import ListView, DetailView
from django.template import loader
from django.template.context import RequestContext
from django.shortcuts import render_to_response

from jmb.mailer.models import EmailObject
from jmb.newsletter.models import Newsletter
from jmb.newsletter2.models import Newsletter as Newsletter2


class NewsletterTypeMixin(object):
    def get_object(self, queryset=None):
        obj = super(NewsletterTypeMixin, self).get_object(queryset)
        if isinstance(obj, Newsletter):
            setattr(obj, 'nl_type', 1)
        elif isinstance(obj, Newsletter2):
            setattr(obj, 'nl_type', 1)
        return obj

class NewsletterDetailView(NewsletterTypeMixin, DetailView):
    context_object_name = 'newsletter_newsletter_detail'
    template_name = 'newsletter/cms_apps/newsletter_detail.html'
    model = Newsletter


class Newsletter2DetailView(NewsletterTypeMixin, DetailView):
    context_object_name = 'newsletter_newsletter2_detail'
    template_name = 'newsletter/cms_apps/newsletter2_detail.html'
    model = Newsletter2

    def get_object(self, queryset=None):
        obj = super(Newsletter2DetailView, self).get_object(queryset)
        return obj

    def get_context_data(self, *args, **kwargs):
        obj = self.get_object()
        context = RequestContext(self.request)
        context['newsletter'] = obj
        template = loader.get_template(settings.PROJECT_PATH+obj.template.file.url)
        context['newsletter_body'] = template.render(context)
        return context