"""Mailer to send newsletter"""
import datetime
from django.template import Template, Context
from django.conf import settings

from jmb.core.utils.functions import slugify
from jmb.mailer.models import EmailObject, SMTPServer, BounceServer

class Mailer(object):
    def __init__(self, newsletter, test=False, verbose=0, recipients=None):
        self.test = test
        self.verbose = verbose
        self.newsletter = newsletter
        self.newsletter_template = Template(self.newsletter.content)
        self.recipients = recipients # list of Contacts

    def run(self):
        """Send the newsletter"""

        # se non definisco alcun destinatario della newsletter
        subscribers = self.recipients
        if not self.recipients:
            mailing_list = self.newsletter.mailing_list
            if self.test:
                """ le email vengono mandate ai sottoscrittori di test"""
                subscribers = mailing_list.get_test_subscribers()
            else:
                """ le email vengono mandate a tutti i sottoscrittori attivi"""
                subscribers = mailing_list.get_subscribers()

        for subscriber in subscribers:
            try:
                email_object = EmailObject()
                if self.test:
                    """ lo stato dell'email e' 'sent test' """
                    email_object.status = settings.MAILER_EMAIL_STATUS_TO_SEND_IN_TEST
                else:
                    """ lo stato dell'email e' 'to send' """
                    email_object.status = settings.MAILER_EMAIL_STATUS_TO_SEND
                email_object.email_address = subscriber.email
                email_object.subject = self.newsletter.title
                context = Context({
                    'newsletter': self.newsletter,
                    'subscriber': subscriber
                })
                email_object.content_object = self.newsletter
                email_object.content = self.newsletter_template.render(context)
                email_object.slug = "%s_%s" % (
                    datetime.datetime.now().strftime("%Y%m%d_%H%M%s"),
                    subscriber.id
                    )
                email_object.smtp_server = SMTPServer.objects.get()
                email_object.bounce_server = BounceServer.objects.get()
                email_object.save()
            except Exception, e:
                print e

    @property
    def can_send(self):
        """Check if the newsletter can be sent"""
        # TOFIX
        return True