"""Utils for importation of contacts"""
import csv
from datetime import datetime

import xlrd
import vobject

from django.core.validators import validate_email
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _


from jmb.newsletter.models import Subscriber, MailingList


COLUMNS = ['email', 'mailinglist_id']
csv.register_dialect('edn', delimiter=';')


def create_contact(contact_dict):
    """
    Create a contact and validate the mail
    """
    contact_dict['email'] = contact_dict['email'].strip()
    try:
        validate_email(contact_dict['email'])
        contact_dict['valid'] = True
    except ValidationError:
        contact_dict['valid'] = False

    contact, created = Subscriber.objects.get_or_create(
        email=contact_dict['email'],
        defaults=contact_dict)

#    if not created:
#        new_tags = contact_dict.get('tags')
#        if new_tags:
#            Tag.objects.update_tags(contact, '%s, %s' % (contact.tags, new_tags))

    return contact, created


def create_contacts(contact_dicts, importer_name):
    """
    Create all the contacts to import and
    associated them in a mailing list
    """
    #import ipdb; ipdb.set_trace()
    inserted = 0
#    when = str(datetime.now()).split('.')[0]
#    mailing_list = MailingList(
#        name=_('Mailing list created by importation at %s') % when,
#        description=_('Contacts imported by %s.') % importer_name)
#    mailing_list.save()
#
    mailing_list_created = None
    for contact_dict in contact_dicts:
        if not contact_dict.has_key('email'):
            continue
        email = contact_dict['email']
        if contact_dict.has_key('mailinglist_id'):
            mailinglist_id = int(contact_dict['mailinglist_id'])
            """
            se viene passato id della mailing_list il contatto viene
            aggiunto alla mailing_list.
            Nel caso in cui la mailing_list non esiste allora non inserisco
            il contatto
            """
            try:
                mailing_list = MailingList.objects.get(id=mailinglist_id)
                print mailing_list
            except:
                print "ml inesistente"
                continue
            try:
                contact = Subscriber(
                    email=email,
                    mailinglist=mailing_list,
                    content_object=mailing_list
                )
                contact.save()
                inserted += 1
                print contact
            except:
                print "esiste gia c in mle"
                continue
        else:
            """
            se non viene passato id della mailing_list allora ne creo una nuova
            """
            if not mailing_list_created:
                """
                se la mailing_list_created non esiste la creo
                """
                when = str(datetime.now()).split('.')[0]
                mailing_list_created= MailingList(
                    name=_('Mailing list created by importation at %s') % when,
                    description=_('Contacts imported by %s.') % importer_name
                )
                mailing_list_created.save()
                print mailing_list_created
            try:
                contact = Subscriber(
                    email=email,
                    mailinglist=mailing_list_created,
                    content_object=mailing_list_created
                )
                contact.save()
                inserted += 1
                print contact
            except:
                print "esiste gia c in ml"
                continue
    return inserted


def text_contacts_import(stream):
    """
    Import contact from a plaintext file, like CSV
    """
    contacts = []
    contact_reader = csv.reader(stream, dialect='edn')

    for contact_row in contact_reader:
        contact = {}
        for i in range(len(contact_row)):
            contact[COLUMNS[i]] = contact_row[i]
        contacts.append(contact)

    return create_contacts(contacts, 'text')


def excel_contacts_import(stream):
    """Import contacts from an Excel file"""
    contacts = []
    wb = xlrd.open_workbook(file_contents=stream.read())
    sh = wb.sheet_by_index(0)

    for row in range(sh.nrows):
        contact = {}
        for i in range(len(COLUMNS)):
            try:
                value = sh.cell(row, i).value
                contact[COLUMNS[i]] = value
            except IndexError:
                break
        contacts.append(contact)

    return create_contacts(contacts, 'excel')


def import_dispatcher(source, type_):
    """
    Select importer and import contacts
    """
    if type_ == 'text':
        return text_contacts_import(source)
    elif type_ == 'excel':
        return excel_contacts_import(source)
    return 0
