"""Tokens system for emails"""
from django.conf import settings
from django.http import Http404
from django.utils.http import int_to_base36, base36_to_int

from jmb.mailer.models import EmailObject


class TokenGenerator(object):
    """Tokengenerator for the email
    based on the PasswordResetTokenGenerator bundled
    in django.contrib.auth"""

    def make_token(self, email):
        """Method for generating the token"""
        from django.utils.hashcompat import sha_constructor

        token = sha_constructor(
            settings.SECRET_KEY + unicode(email.id) +
            email.email_address).hexdigest()[::2]
        return token

    def check_token(self, email, token):
        """Check if the token is correct for this user"""
        return token == self.make_token(email)


def tokenize(email):
    """Return the uid in base 36 of an email and a token"""
    token_generator = TokenGenerator()
    return int_to_base36(email.id), token_generator.make_token(email)


def untokenize(uidb36, token):
    """Retrieve an email by uidb36 and token"""
    try:
        email_id = base36_to_int(uidb36)
        email = EmailObject.objects.get(pk=email_id)
    except:
        raise Http404

    token_generator = TokenGenerator()
    if token_generator.check_token(email, token):
        return email
    raise Http404