#! -*- coding: utf-8 -*-

import csv, sys
from datetime import timedelta, datetime, date
import base64
from urllib import urlencode
from urlparse import urlparse
from urlparse import urlunparse
# For Python < 2.6
from jmb.newsletter.forms import MailingListUnsubscriptionForm

try:
    from urlparse import parse_qs
except ImportError:
    from cgi import parse_qs

from django.template import RequestContext
from django.shortcuts import get_object_or_404
from django.shortcuts import render_to_response
from django.contrib.sites.models import Site
from django.contrib.auth.decorators import login_required
from django.template.loader import render_to_string as render_file
from django.db.models import Q
from django.http import HttpResponse, Http404
from django.http import HttpResponseRedirect
from django.utils.encoding import smart_str
from django.utils.translation import ugettext as _
from django.template.defaultfilters import date
from django.utils.encoding import smart_str
from django.contrib.contenttypes.models import ContentType
from django.conf import settings

from jmb.core.utils.charts import Chart, Property, SubProperty

from jmb.mailer.statistics import *
from jmb.newsletter.models import Newsletter
from jmb.newsletter.models import MailingList
from jmb.newsletter.models import Subscriber

from jmb.newsletter.utils import render_string
#from jmb.newsletter.utils.nl_utils import body_insertion
#from newsletter.utils.nl_utils import track_links
from jmb.newsletter.utils.tokens import untokenize
from jmb.mailer.utils.tokens import untokenize as mailer_untokenize
#from newsletter.utils.ofc import Chart
#from newsletter.settings import USE_UTM_TAGS
#from newsletter.settings import TRACKING_IMAGE
#from newsletter.utils.statistics import get_newsletter_top_links
#from newsletter.utils.statistics import get_newsletter_statistics
#from newsletter.utils.statistics import get_newsletter_opening_statistics
#from newsletter.utils.statistics import get_newsletter_clicked_link_statistics
#from newsletter.forms import CreditForm

@login_required
def view_newsletter_preview(request, slug):
    """View of the newsletter preview"""
    pass
#    context = {'contact': request.user}
#    return render_newsletter(request, slug, context)

def view_newsletter_contact(request, slug, uidb36, token):
    """Visualization of a newsletter by an user"""
    newsletter = get_object_or_404(Newsletter, slug=slug)
    contact = untokenize(uidb36, token)
#    try:
#        cms = ContactMailingStatus.objects.filter(
#            newsletter=newsletter,
#            contact=contact,
#            status=0
#        ).order_by('-id')
#        if cms:
#            cms = cms[0]
#        cms.opened_on_site = cms.opened_on_site + 1
#        if not cms.first_open_date:
#            cms.first_open_date = datetime.now()
#        cms.save()
#    except:
#        raise Http404
#
#    context = {
#        'contact': contact, 'uidb36': uidb36, 'token': token
#    }
#
#    return render_newsletter(request, slug, context)

# MAILING LIST

def all_mailinglist_unsubscribe(request,uidb36, token):
    """Unsubscribe an email from a mailing list"""

    email_object = mailer_untokenize(uidb36, token)
    step = 'start'
    if email_object:
        # prende tutti i sottoscrittori attivi con l'email uguale a quella ricavata
        active_subscribers = Subscriber.objects.filter(
            date_unsubscription__isnull=True,
            email=email_object.email_address
        )
    else:
        raise Http404

    if request.POST and active_subscribers:
        for active_subscriber in active_subscribers:
            active_subscriber.date_unsubscription = datetime.now()
            active_subscriber.save()
        step = 'finish'
    if not active_subscribers:
        step = 'error'

    return render_to_response(
        'newsletter/all_mailing_lists_unsubscribe.html', {
            'email_object':  email_object,
            'step':          step,
            },
        context_instance=RequestContext(request)
    )


def mailinglist_unsubscribe(request,uidb36, token):
    """Unsubscribe an email from a mailing list"""
    email_object = mailer_untokenize(uidb36, token)
    step = 'start'
    mailing_list_form = None

    if email_object:
        # prende tutti i sottoscrittori attivi con l'email uguale a quella ricavata
        active_subscribers = Subscriber.objects.filter(
            date_unsubscription__isnull=True,
            email=email_object.email_address
        )
    else:
        raise Http404

    if not active_subscribers:
        step = 'error'

    if request.POST and active_subscribers:
        mailing_list_form = MailingListUnsubscriptionForm(active_subscribers, request.POST)
        if mailing_list_form.is_valid():
            mailing_list_form.save()
    if not request.POST and active_subscribers:
        mailing_list_form = MailingListUnsubscriptionForm(active_subscribers)
    return render_to_response(
        'newsletter/mailing_list_unsubscribe.html', {
            'email_object':         email_object,
            'mailing_list_form':    mailing_list_form,
            'step':                 step,
            },
        context_instance=RequestContext(request)
    )

def view_mailinglist_subscribe(request, form_class, mailing_list_id=None):
    """
    A simple view that shows a form for subscription
    for a mailing list(s).
    """
    subscribed = False
    mailing_list = None
    if mailing_list_id:
        mailing_list = get_object_or_404(MailingList, id=mailing_list_id)

    if request.POST and not subscribed:
        form = form_class(request.POST)
        if form.is_valid():
            form.save(mailing_list)
            subscribed = True
    else:
        form = form_class()

    return render_to_response(
        'newsletter/mailing_list_subscribe.html',
        {'subscribed': subscribed, 'mailing_list': mailing_list, 'form': form},
        context_instance=RequestContext(request)
    )

def newsletter_statistics(request, slug):
    """Display statistics of newsletter"""
    opts = Newsletter._meta
    newsletter = get_object_or_404(Newsletter, slug=slug)
    newsletter_type = ContentType.objects.get_for_model(newsletter)

    from dateutil.rrule import rrule, DAILY
    from jmb.mailer.statistics import Statistic

    statistic = Statistic(
        object_id=newsletter.id,
        content_type_id=newsletter_type.id
    )
    statistic.recalculate()

    stats = {}

    stats['all_subscribers'] = Subscriber.objects.filter(
        date_unsubscription__isnull=True
    ).count()
    stats['all_unsubscribers'] = Subscriber.objects.filter(
        date_unsubscription__isnull=False
    ).count()

    stats['total_sum_sent_in_test'] = statistic.sum_per_status.get(settings.MAILER_MAILINGSTATUS_STATUS_SENT_TEST, 0)
    stats['total_unique_sent_in_test'] = statistic.unique_per_status.get(settings.MAILER_MAILINGSTATUS_STATUS_SENT_TEST, 0)
    stats['total_sum_to_send'] = statistic.sum_per_status.get(settings.MAILER_MAILINGSTATUS_STATUS_TO_SEND, 0)
    stats['total_unique_to_send'] = statistic.unique_per_status.get(settings.MAILER_MAILINGSTATUS_STATUS_TO_SEND, 0)
    stats['total_sum_sent'] = statistic.sum_per_status.get(settings.MAILER_MAILINGSTATUS_STATUS_SENT, 0)
    stats['total_unique_sent'] = statistic.unique_per_status.get(settings.MAILER_MAILINGSTATUS_STATUS_SENT, 0)

    stats['total_sum_opened'] = statistic.sum_per_status.get(settings.MAILER_MAILINGSTATUS_STATUS_OPENED, 0)
    stats['total_unique_opened'] = statistic.unique_per_status.get(settings.MAILER_MAILINGSTATUS_STATUS_OPENED, 0)
    stats['total_sum_opened_on_site'] = statistic.sum_per_status.get(settings.MAILER_MAILINGSTATUS_STATUS_OPENED_ON_SITE, 0)
    stats['total_unique_opened_on_site'] = statistic.unique_per_status.get(settings.MAILER_MAILINGSTATUS_STATUS_OPENED_ON_SITE, 0)
    stats['total_sum_link_opened'] = statistic.sum_per_status.get(settings.MAILER_MAILINGSTATUS_STATUS_LINK_OPENED, 0)
    stats['total_unique_link_opened'] = statistic.unique_per_status.get(settings.MAILER_MAILINGSTATUS_STATUS_LINK_OPENED, 0)

    # LINKS DATA
    stats['links'] = []
    for ind, link in enumerate(statistic.sum_per_link):
        stats['links'].append((
            link,
            statistic.sum_per_link.get(str(link), 0),
            statistic.unique_per_link.get(str(link), 0)
            ))

    stats['total_sum_error'] = statistic.sum_per_status.get(settings.MAILER_MAILINGSTATUS_STATUS_ERROR, 0)
    stats['total_unique_error'] = statistic.unique_per_status.get(settings.MAILER_MAILINGSTATUS_STATUS_ERROR, 0)
    stats['total_sum_invalid'] = statistic.sum_per_status.get(settings.MAILER_MAILINGSTATUS_STATUS_INVALID, 0)
    stats['total_unique_invalid'] = statistic.unique_per_status.get(settings.MAILER_MAILINGSTATUS_STATUS_INVALID, 0)
    stats['total_sum_bounced'] = statistic.sum_per_status.get(settings.MAILER_MAILINGSTATUS_STATUS_BOUNCED, 0)
    stats['total_unique_bounced'] = statistic.unique_per_status.get(settings.MAILER_MAILINGSTATUS_STATUS_BOUNCED, 0)

    stats['total_sum_delivered'] = stats['total_sum_sent'] - (
        stats['total_sum_error'] + stats['total_sum_invalid'] + stats['total_sum_bounced']
        )
    stats['total_unique_delivered'] = stats['total_unique_sent'] - (
        stats['total_unique_error'] + stats['total_unique_invalid'] + stats['total_unique_bounced']
        )

    stats['total_unsubscriptions'] = newsletter.unsubscriptions

    # CREO GRAFICI
    date_start = statistic.range[0]
    date_end = statistic.range[1]
    line_chart = {}

    # GRAFICO 'DETTAGLIO INVIATE'
    # ------------------------------------------------------------------------------------------------------------------
    labels = []
    values = {}

    line_chart['sent_aggregate'] = Chart('sent_aggregate', defaultSeriesType='line')
    line_chart['sent_aggregate'].add_property('title', text='DETTAGLIO INVII')
    line_chart['sent_aggregate'].add_property('subtitle', text='Statistiche Dettaglio Invii')
    line_chart['sent_aggregate'].add_property('yAxis', title=SubProperty(text='Quantita'))
    line_chart['sent_aggregate'].add_property('tooltip', crosshairs=True, shared=True)

    status_list = (
        settings.MAILER_MAILINGSTATUS_STATUS_SENT,
        settings.MAILER_MAILINGSTATUS_STATUS_OPENED,
        settings.MAILER_MAILINGSTATUS_STATUS_OPENED_ON_SITE,
        settings.MAILER_MAILINGSTATUS_STATUS_LINK_OPENED
    )
    for date in rrule(DAILY, dtstart=date_start, until=date_end):
        str_date = date.strftime('%Y%m%d')
        labels.append(str_date)
        # ciclo la lista degli stati
        for status in status_list:
            if not values.get(status):
                values[status] = []
            if statistic.sum_per_status_per_day.has_key((status, str_date)):
                values[status].append(statistic.sum_per_status_per_day[(status,str_date)])
            else:
                values[status].append(0)

    line_chart['sent_aggregate'].add_property('xAxis', categories=labels)
    line_chart['sent_aggregate'].add_series(name='Total SENT Mails', data=values[settings.MAILER_MAILINGSTATUS_STATUS_SENT])
    line_chart['sent_aggregate'].add_series(name='Total OPENED Mails', data=values[settings.MAILER_MAILINGSTATUS_STATUS_OPENED])
    line_chart['sent_aggregate'].add_series(name='Total OPENED ON SITE Mails', data=values[settings.MAILER_MAILINGSTATUS_STATUS_OPENED_ON_SITE])
    line_chart['sent_aggregate'].add_series(name='Total OPENED Links', data=values[settings.MAILER_MAILINGSTATUS_STATUS_LINK_OPENED])

    # GRAFICO 'DETTAGLIO APERTURE'
    # ------------------------------------------------------------------------------------------------------------------
    labels = []

    line_chart['open_aggregate'] = Chart('open_aggregate', defaultSeriesType='line')
    line_chart['open_aggregate'].add_property('title', text='DETTAGLIO APERTURE')
    line_chart['open_aggregate'].add_property('subtitle', text='Statistiche Dettaglio Aperture')
    line_chart['open_aggregate'].add_property('yAxis', title=SubProperty(text='Quantita'))
    line_chart['open_aggregate'].add_property('tooltip', crosshairs=True, shared=True)

    sum_opened_values = []
    unique_opened_values = []

    for date in rrule(DAILY, dtstart=date_start, until=date_end):
        str_date = date.strftime('%Y%m%d')
        labels.append(str_date)
        # ciclo la lista degli stati

        # SUM OPENED
        status = settings.MAILER_MAILINGSTATUS_STATUS_OPENED
        if statistic.sum_per_status_per_day.has_key((status, str_date)):
            opened = statistic.sum_per_status_per_day[(status,str_date)]
        else:
            opened = 0
            # SUM OPENED ON SITE
        status = settings.MAILER_MAILINGSTATUS_STATUS_OPENED_ON_SITE
        if statistic.sum_per_status_per_day.has_key((status, str_date)):
            opened_on_site = statistic.sum_per_status_per_day[(status,str_date)]
        else:
            opened_on_site = 0
        sum_opened_values.append(opened+opened_on_site)

        # UNIQUE OPENED
        status = settings.MAILER_MAILINGSTATUS_STATUS_OPENED
        if statistic.unique_per_status_per_day.has_key((status, str_date)):
            opened = statistic.unique_per_status_per_day[(status,str_date)]
        else:
            opened = 0
            # UNIQUE OPENED ON SITE
        status = settings.MAILER_MAILINGSTATUS_STATUS_OPENED_ON_SITE
        if statistic.unique_per_status_per_day.has_key((status, str_date)):
            opened_on_site = statistic.unique_per_status_per_day[(status,str_date)]
        else:
            opened_on_site = 0
        unique_opened_values.append(opened+opened_on_site)

    line_chart['open_aggregate'].add_property('xAxis', categories=labels)
    line_chart['open_aggregate'].add_series(name='Total OPENED and OPENED ON SITE', data=sum_opened_values)
    line_chart['open_aggregate'].add_series(name='Unique OPENED and OPENED ON SITE', data=unique_opened_values)

    # GRAFICO 'DETTAGLIO LINK'
    # ------------------------------------------------------------------------------------------------------------------
    labels = []

    line_chart['link_aggregate'] = Chart('link_aggregate', defaultSeriesType='column')
    line_chart['link_aggregate'].add_property('title', text='DETTAGLIO LINK')
    line_chart['link_aggregate'].add_property('subtitle', text='Statistiche Dettaglio Link')
    line_chart['link_aggregate'].add_property('yAxis', title=SubProperty(text='Quantita'))
    line_chart['link_aggregate'].add_property('tooltip', crosshairs=True, shared=True)

    sum_link_values = []
    unique_link_values = []
    sum_sing_link_values = {}
    links = []

    for date in rrule(DAILY, dtstart=date_start, until=date_end):
        str_date = date.strftime('%Y%m%d')
        labels.append(str_date)
        # ciclo la lista degli stati

        # SUM OPENED LINKS
        status = settings.MAILER_MAILINGSTATUS_STATUS_LINK_OPENED
        if statistic.sum_per_status_per_day.has_key((status, str_date)):
            sum_link_values.append(statistic.sum_per_status_per_day[(status,str_date)])
        else:
            sum_link_values.append(0)

        # UNIQUE OPENED LINKS
        if statistic.unique_per_status_per_day.has_key((status, str_date)):
            unique_link_values.append(statistic.unique_per_status_per_day[(status,str_date)])
        else:
            unique_link_values.append(0)

        # SUM OPENED PER LINKS
        for link in statistic.sum_per_link:
            value = statistic.sum_per_link_per_day.get((link, str_date), 0)
            if not sum_sing_link_values.get(link):
                sum_sing_link_values[link] = [value]
                links.append(link)
            else:
                sum_sing_link_values[link].append(value)

    # per ogni link aggiunge la serie
    for link in links:
        values = sum_sing_link_values[link]
        line_chart['link_aggregate'].add_series(name=str(link), data=values)

    line_chart['link_aggregate'].add_property('xAxis', categories=labels)
    line_chart['link_aggregate'].add_series(name='Total OPENED Links', data=sum_link_values)
    line_chart['link_aggregate'].add_series(name='Unique OPENED Links', data=unique_link_values)

    context = {
        'title': _('Historic of %s') % newsletter.__unicode__(),
        'newsletter': newsletter,
        'opts': opts,
        'object_id': newsletter.pk,
        'app_label': opts.app_label,
        'stats': stats,
        'links': links,
        'line_chart': line_chart,
        }
    return render_to_response(
        'admin/newsletter/newsletter_statistics.html',
        context, context_instance=RequestContext(request)
    )